/* 
   The code in this file relies heavily on compiler optimization.  The
   code was written on an x86 with GCC 2.95 19990728, and using -O2 it
   produced assembly code comparable to hand-crafted assembly. Without
   optimiation the generated code was truly awful.

   The goal is to produce an optimized flat loop without any kind of
   conditional execution. All functions are merged and all constant
   expressions are evaluated during compile-time.

 */


typedef unsigned char byte;
typedef byte* bytep;
typedef unsigned short uint16;
typedef unsigned int   uint32;

class ExtractRGB888 {
public:
  typedef byte channelValue;
  static const bool hasAlpha = false;
  static const int rh = 8;
  static const int gh = 8;
  static const int bh = 8;
};

class ExtractRGBa8888 {
public:
  typedef byte channelValue;
  static const bool hasAlpha = true;
  static const int rh = 8;
  static const int gh = 8;
  static const int bh = 8;
};

class PackRGB565 {
public:
  static const int rh = 16;
  static const int rw = 5;
  static const int gh = 11;
  static const int gw = 6;
  static const int bh = 5;
  static const int bw = 5;
  typedef byte   inT;
  typedef uint16 outT;
};

class PackXRGB1555 {
public:
  static const int rh = 15;
  static const int rw = 5;
  static const int gh = 10;
  static const int gw = 5;
  static const int bh = 5;
  static const int bw = 5;
  typedef byte   inT;
  typedef uint16 outT;
};



template<class outT, class inT>
inline outT isolateAndShift(inT v, int highInBit, int highOutBit, int width) {

  // inT:   ??????????xxxxxxx??????
  // outT:  000000xxxxxxx0000000000

  int inTWidth = sizeof(inT)*8;
  // int outTWidth = sizeof(outT)*8;
  int leftDiscardShift = inTWidth - highInBit;
  int rightDiscardShift = inTWidth - width;
  int finalLeftShift = highOutBit-width;
  v <<= leftDiscardShift;
  v >>= rightDiscardShift;
  outT stripped = v;
  return stripped << finalLeftShift;
}

template<int counter, class Part, class Whole>
inline Part consume(bytep& in) {
  int partsPerWhole = sizeof(Whole) / sizeof(Part);
  int bitsPerPart = sizeof(Part)*8;
  int bytesPerWhole = sizeof(Whole);
  int phase = counter % partsPerWhole;

  Whole* wholeIn = (Whole*) in;
  Part ret = ((*wholeIn) >> (phase*bitsPerPart)) & 0xff;
  if ((phase+1) == partsPerWhole) in += bytesPerWhole;
  return ret;
}

template<int counter, class Extract, class unit>
inline void consumeRGBA(bytep& in,
			typename Extract::channelValue& r,
			typename Extract::channelValue& g,
			typename Extract::channelValue& b,
			typename Extract::channelValue& a
) {
  
  /* This can be expanded to support several types of extraction
     methods. */
  const int channels = Extract::hasAlpha ? 4 : 3;

  // FIXME: We currently assume RGB888 or aRaGaBa??????? format...
  b = consume<counter*channels+0, typename Extract::channelValue, unit>(in);
  g = consume<counter*channels+1, typename Extract::channelValue, unit>(in);
  r = consume<counter*channels+2, typename Extract::channelValue, unit>(in);
  if (Extract::hasAlpha) {
    a = consume<counter*channels+3, typename Extract::channelValue, unit>(in);
  }
}

template <int counter, class Part, class Whole>
inline void emit(bytep& out, Part v, Whole& buffer) {
  int partsPerWhole = sizeof(Whole) / sizeof(Part);
  int bitsPerPart = sizeof(Part)*8;
  int bytesPerWhole = sizeof(Whole);
  int phase = counter % partsPerWhole;

  buffer |= v << (phase*bitsPerPart);
  if ((phase+1) == partsPerWhole) {
    Whole* wholeOut = (Whole*) out;
    *wholeOut = buffer;
    buffer = 0;
    out += bytesPerWhole;
  }
}

template<int counter, class Extract, class Pack, class unit>
inline void spool(bytep& in, bytep& out, uint32& buffer) {
  
  typedef typename Pack::outT outT;
  typedef typename Extract::channelValue channelValue;

  channelValue r, g, b, a;
  consumeRGBA<counter,Extract,unit>(in, r, g, b, a);
  outT pixel = 
    isolateAndShift<outT,byte>(r, Extract::rh, Pack::rh, Pack::rw) |
    isolateAndShift<outT,byte>(g, Extract::gh, Pack::gh, Pack::gw) |
    isolateAndShift<outT,byte>(b, Extract::bh, Pack::bh, Pack::bw);
  emit<counter, outT, uint32>(out, pixel, buffer);
}
    
template <class Extract, class Pack, class unit>
inline void render(byte in[], byte out[], int width, int inBytesPerLine,
		    int outBytesPerLine, int height) {

    const int chunkSize = sizeof(unit);
    const int xChunks = inBytesPerLine / chunkSize;
    
    for (int y=0; y<height; y++) {
	byte*  inRow = in;
	byte* outRow = out;
	
	for (int xc=0; xc<xChunks; xc++) {
	    uint32 buffer = 0;
	    if (chunkSize>0) spool<0,Extract,Pack,unit>(inRow, outRow, buffer);
	    if (chunkSize>1) spool<1,Extract,Pack,unit>(inRow, outRow, buffer);
	    if (chunkSize>2) spool<2,Extract,Pack,unit>(inRow, outRow, buffer);
	    if (chunkSize>3) spool<3,Extract,Pack,unit>(inRow, outRow, buffer);
	    if (chunkSize>4) spool<4,Extract,Pack,unit>(inRow, outRow, buffer);
	    if (chunkSize>5) spool<5,Extract,Pack,unit>(inRow, outRow, buffer);
	    if (chunkSize>6) spool<6,Extract,Pack,unit>(inRow, outRow, buffer);
	    if (chunkSize>7) spool<7,Extract,Pack,unit>(inRow, outRow, buffer);
	}

	in  +=  inBytesPerLine;
	out += outBytesPerLine;
    }
}

