package no.rwr.util;

public class ArrayUtil {
    public static boolean equals(int[] a, int[] b) {
	int length = a.length;
	if (length != b.length) return false;
	for (int i=0; i<length; i++)
	    if (a[i] != b[i]) return false;
	return true;
    }


    /** Calculate the inverse mapping of a mapping given by an int
        array representing a look up table.

	<p>Note: We are talking about the <em>inverse</em>, not the
	<em>reverse</em> here.

	<p>Eg. {3, 1, 0, 2} returns {2, 1, 3, 0}:

	<pre>
	*   0 ==> 3 ---.    ,---- 0 ==> 2 ----.    ,--- 0
	*               \  /                   \  /
	*   1 ==> 1 -----\/------ 1 ==> 1 ------\/----- 1
	*                /\                     /\ 
	*   2 ==> 0 ----'  \,---- 2 ==> 3 ----./  `---- 2
	*                  /\                 /\   
	*   3 ==> 2 ------'  `--- 3 ==> 0 ---'  `------ 3
	</pre>

	@return array containing the reverse mapping */

    public static int[] invert(int[] mapping, int[] returnArray) {	
	int size = mapping.length;

	if (returnArray == null) returnArray = new int[size];

	for (int i=0; i<size; i++) returnArray[mapping[i]] = i;
	return returnArray;
    }  

    public static boolean isCompleteInverse(int[] mapping,
					    int[] inverseMapping) {
	int size = mapping.length;
	
	for (int i=0; i<size; i++)
	    if (inverseMapping[mapping[i]] != i) return false;

	return true;
    }
}
