package no.rwr.x;

import gnu.gcj.RawData;

/**

   Screen is a lightweight class. Display and screen number is the
   only data, The real structure is stored in the display.

   There may exist several objects denoting the same screen.
 */

public final class Screen {
    Display display;
    int screenNumber;

    public Screen(Display display) {
	this(display, display.getDefaultScreenNumber());
    }

    public Screen(Display display, int screenNumber) {
	this.display = display;
	this.screenNumber = screenNumber;
    }
    
    public final Display getDisplay() {
	return display;
    }

    public Window getRootWindow() {
	int rootXID = getRootWindowXID();
	return display.getWindow(rootXID);
    }

    public Visual getRootVisual() {
	RawData visualStructure = getRootVisualStructure();
	int depth = getRootDepth();
	return new Visual(visualStructure, this, depth);
    }

    private native RawData getRootVisualStructure();

    public native int getRootDepth();
    public native int getRootWindowXID();
    public native int getDefaultColormapXID();

    public Colormap getDefaultColormap() {
	return new Colormap(display, getDefaultColormapXID());
    }

    public final int getScreenNumber() {
	return screenNumber;
    }
}
