
#include <X11/Xlib.h>

#include <cni.h>
#include "no/rwr/x/Display.h"
#include "no/rwr/x/ConnectionException.h"

void no::rwr::x::Display::init() {
    ::Display* openedDisplay = XOpenDisplay(0); // default display

    if (openedDisplay == 0) {
	jstring msg = JvNewStringLatin1("Unable to open display");
	JvThrow(new no::rwr::x::ConnectionException(msg));
    }

    display = reinterpret_cast<gnu::gcj::RawData*>(openedDisplay); 
}

void no::rwr::x::Display::finalize() {
    if (display == 0) return;
    ::Display* dpy = (::Display*) display;
    XCloseDisplay(dpy);
}

jint no::rwr::x::Display::getDefaultScreenNumber() {
    ::Display* dpy = (::Display*) display;
    return DefaultScreen(dpy);
}

jint no::rwr::x::Display::getDefaultRootWindowXID() {
    ::Display* dpy = (::Display*) display;
    return DefaultRootWindow(dpy);
}

jint no::rwr::x::Display::internAtom(jstring name) {
    ::Display* dpy = (::Display*) display;
    int len = JvGetStringUTFLength(name);
    char cName[len+1];
    JvGetStringUTFRegion(name, 0, name->length(), cName);
    cName[len] = '\0';
    bool onlyIfExists = false;
    return XInternAtom(dpy, cName, onlyIfExists);
}

jstring no::rwr::x::Display::getAtomName(jint atom) {
    ::Display* dpy = (::Display*) display;
    char* cName = XGetAtomName(dpy, atom);
    jstring name = JvNewStringLatin1(cName);
    XFree(cName);
    return name;
}

