
#include <X11/Xlib.h>

#include <cni.h>
#include <gnu/gcj/RawData.h>
#include <java/lang/String.h>


#include "no/rwr/x/Display.h"
#include "no/rwr/x/Drawable.h"
#include "no/rwr/x/Font.h"
#include "no/rwr/x/GC.h"
#include "no/rwr/x/XImage.h"

gnu::gcj::RawData* no::rwr::x::GC::getValues() {
    if (values == 0) {
	XGCValues* vals = new XGCValues;
	values = reinterpret_cast<gnu::gcj::RawData*>(vals);
    }

    return values;
}

gnu::gcj::RawData* no::rwr::x::GC::getStructure() {

    // Perform lazy creation or updating if necessary
    if (structure == 0) {
	Display* display = targetType->getDisplay();
	::Display* dpy = (::Display*) (display->display);
	::Drawable drawableXID = targetType->getXID();

	/* We don't use getValues() since we don't care that vals ==
	   0, since if vals == 0 then valueMask == 0.  */
	::XGCValues* vals = (::XGCValues*) values;

	::GC gc = XCreateGC(dpy, drawableXID, valueMask, vals);
	structure = reinterpret_cast<gnu::gcj::RawData*>(gc);

	valueMask = 0; // All values are now up to date
    } else if (valueMask != 0) {	
	Display* display = targetType->getDisplay();
	::Display* dpy = (::Display*) (display->display);
	::XGCValues* vals = (::XGCValues*) values;
	::GC gc = (::GC) structure;
	XChangeGC(dpy, gc, valueMask, vals);
	valueMask = 0; // All values are now up to date
    }

    return structure;
}

void no::rwr::x::GC::dispose() {
    if (structure != 0) {
	Display* display = targetType->getDisplay();
	::Display* dpy = (::Display*) (display->display);
	::GC gc = (::GC) structure;
	XFreeGC(dpy, gc);
	structure = 0;
    }
    if (values != 0) {
	delete values;
	values = 0;
    }
}

void no::rwr::x::GC::setForeground(jlong pixel) {
    ::XGCValues* vals = (::XGCValues*) getValues();;

    vals->foreground = pixel;
    valueMask = valueMask | GCForeground;
}

void no::rwr::x::GC::setFont(Font* font) {
    ::XGCValues* vals = (::XGCValues*) getValues();

    vals->font = font->getXID();
    valueMask = valueMask | GCFont;
}

void no::rwr::x::GC::drawString(jstring text, jint x, jint y) {
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    ::GC gc = (::GC) getStructure();

    /*
    jint length = text->length();
    jchar* txt = JvGetStringChars(text);

    XChar2b xwchars[length];
    
    // FIXME: Add convertion and caching

    for (int i=0; i<length; i++) {
	XChar2b* xc = &(xwchars[i]);
	jchar jc = txt[i];
	xc->byte1 = jc & 0xff;
	xc->byte2 = jc >> 8;
    }

     XDrawString16(dpy, drawableXID, gc, x, y, xwchars, length);
    */

    // ALTERNATIVE:
    int len = JvGetStringUTFLength(text);
    char ctxt[len+1];
    JvGetStringUTFRegion(text, 0, text->length(), ctxt);
    ctxt[len] = '\0';
    XDrawString(dpy, drawableXID, gc, x, y, ctxt, len);
}

void no::rwr::x::GC::drawLine(jint x1, jint y1, jint x2, jint y2) {
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    ::GC gc = (::GC) getStructure();
    XDrawLine(dpy, drawableXID, gc, x1, y1, x2, y2);
}

void no::rwr::x::GC::drawRectangle(jint x, jint y, jint w, jint h) {
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    ::GC gc = (::GC) getStructure();
    XDrawRectangle(dpy, drawableXID, gc, x, y, w, h);
}

void no::rwr::x::GC::putImage(XImage* image,
			      jint srcX, jint srcY,
			      jint destX, jint destY,
			      jint width, jint height) {
    
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    ::GC gc = (::GC) getStructure();
    ::XImage* ximage = (::XImage*) (image->structure);

    XPutImage(dpy, drawableXID, gc, ximage,
	      srcX, srcY,
	      destX, destY,
	      width, height);
}
