
#include <vector>
#include <config.h>

#include <X11/Xlib.h>

#include <cni.h>
#include <jvm.h>
#include "java/awt/Rectangle.h"
#include "no/rwr/x/Display.h"
#include "no/rwr/x/Window.h"
#include "no/rwr/x/WindowAttributes.h"
#include "no/rwr/x/Visual.h"

jint no::rwr::x::Window::createChildXID(java::awt::Rectangle* bounds,
					jint borderWidth,
					WindowAttributes* attributes,
					jint windowIOClass, 
					Visual* visual) {
    ::Window parentXID = xid;
    
    int x      = bounds->x;
    int y      = bounds->y;
    int width  = bounds->width;
    int height = bounds->height;
    
    long mask = attributes->mask;
    XSetWindowAttributes* attr = (XSetWindowAttributes*) attributes->structure;
    
    ::Visual* vis = CopyFromParent;
    int depth = CopyFromParent;
    if (visual != 0) {
	vis = (::Visual*) visual->getVisualStructure();
	depth = visual->getDepth();
    } 

    ::Window childXID = XCreateWindow((::Display*) (display->display),
				      parentXID,
				      x, y, width, height,
				      borderWidth, depth, windowIOClass,
				      vis,
				      mask, attr);
    return childXID;
}

void no::rwr::x::Window::destroy() {
    ::Display* dpy = (::Display*) (display->display);
    ::Window window = xid;
    XDestroyWindow(dpy, window);
}

void no::rwr::x::Window::setAttributes(WindowAttributes* attributes) {
    ::Display* dpy = (::Display*) (display->display);
    ::Window window = xid;
    ::XSetWindowAttributes* attr =
	(::XSetWindowAttributes*) attributes->structure;

    XChangeWindowAttributes(dpy, window, attributes->mask, attr);
}




void no::rwr::x::Window::map() {
    ::Display* dpy = (::Display*) (display->display);
    ::Window window = xid;
    XMapWindow(dpy, window);
}


void no::rwr::x::Window::setProperty(jint nameAtom, jint typeAtom, 
				     jbyteArray data) {
    ::Display* dpy = (::Display*) (display->display);
    int format = 8;
    int mode = PropModeReplace;
    unsigned char* pData = (unsigned char*) elements(data);
    int len = data->length;
    
    XChangeProperty(dpy, xid, nameAtom, typeAtom, format, mode,
		    pData, len);
}

void no::rwr::x::Window::setWMProtocols(jintArray atoms) {
     ::Display* dpy = (::Display*) (display->display);
     
     size_t length = atoms->length;
     jint* atomsBegin = elements(atoms);
     jint* atomsEnd   = atomsBegin + length;
     vector<Atom> atomVector(atomsBegin, atomsEnd);
     Atom* atomsArray = atomVector.begin();
     XSetWMProtocols(dpy, xid, atomsArray, length);
}
